/*
 * This file is part of SpongeAPI, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.api.data.manipulator.immutable.entity;

import org.spongepowered.api.data.manipulator.ImmutableDataManipulator;
import org.spongepowered.api.data.manipulator.mutable.entity.InvulnerabilityData;
import org.spongepowered.api.data.value.immutable.ImmutableBoundedValue;
import org.spongepowered.api.data.value.immutable.ImmutableValue;
import org.spongepowered.api.entity.Entity;

/**
 * A {@link ImmutableDataManipulator} for the invulnerability an {@link Entity}
 * can have such that any "damage" occurring while the {@link Entity} is
 * "invulnerable" will not actually "damage" the {@link Entity}.
 */
public interface ImmutableInvulnerabilityData extends ImmutableDataManipulator<ImmutableInvulnerabilityData, InvulnerabilityData> {

    /**
     * Gets the {@link ImmutableBoundedValue} for the amount of "ticks" of
     * "invulnerability".
     *
     * @return The immutable value for the amount of ticks of invulnerability
     */
    ImmutableBoundedValue<Integer> invulnerableTicks();

    /**
     * Gets the boolean {@link ImmutableValue} which represents if an entity is
     * invulnerable from most damage sources besides <b>besides</b> the void,
     * players in creative mode, and manual killing like the /kill command.
     *
     * <p>This does not cover creative mode, where players can also
     * be invulnerable.</p>
     *
     * @return The boolean value for whether or not the entity is invulnerable
     */
    ImmutableValue<Boolean> invulnerable();

}
